/*
 * Copyright (C) 2006-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package com.funambol.storage;

import com.funambol.util.Log;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import jmunit.framework.cldc10.AssertionFailedException;
import jmunit.framework.cldc10.TestCase;

public class ComplexSerializerTest extends TestCase {
    /** Private constants suitable for testing*/
    private static final int NULL = 0;
    private static final int INTEGER = 1;
    private static final int STRING = 2;
    private static final int SERIALIZED = 3;
    private static final int BYTE_ARRAY = 4;
    private static final int HASHTABLE = 5;
    private static final int VECTOR = 6;
    
    /** 
     * Private Object to be serialized
     */
    private Integer i;
    private String str;
    private byte[] ba;
    private Hashtable ht;
    private Vector v;
    private Hashtable htVectorHashtable;
    private Vector vht;
    private Hashtable serializableHt;
    
    /** 
     * Private Object to be deserialized
     */
    private byte[] iStream;
    private byte[] strStream;
    private byte[] baStream;
    private byte[] htStream;
    private byte[] vStream;
    private byte[] htvhtStream;
    private byte[] vhtStream;
    private byte[] serializableHtStream;
    
    /**
     * Creates a new instance of this testcase  
     */
    public ComplexSerializerTest() {
        super(16,"ComplexSerializerTest");
    }

    /**
     * TestCase setUp default method  
     */
    public void setUp() {
        createObjects();
    }

    /**
     * TestCase tearDown default method  
     */
    public void tearDown() {
    }

    /**
     * TestCase core with all tests 
     */
    public void test(int testNumber) throws Throwable {
        switch(testNumber) {
            case 0:
                serializeIntegerTest();
                break;
            case 1:
                serializeStringTest();
                break;
            case 2:
                serializeByteArrayTest();
                break;
            case 3:
                testSerializeHashTable();
                break;
            case 4:
                testSerializeVector();
                break;
            case 5:
                testSerializeHashtableOfVector();
                break;
            case 6:
                testSerializeVectorOfVector();
                break;
            case 7:
                testDeserializeInteger();
                break;
            case 8:
                testDeserializeString();
                break;
            case 9:
                testDeserializeByteArray();
                break;
            case 10:
                testDeserializeHashTable();
                break;
            case 11:
                testDeserializeVector();
                break;
            case 12:
                testDeserializeHashtableOfVector();
                break;
            case 13:
                testDeserializeVectorOfVector();
                break;
            case 14:
                testSerializeSerializableHashtable();
                break;
            case 15:
                deserializeSerializableHashtable();
                break;
            default: 
                break;
        }
    }
    
    /**
     * Test of method serializeObject for Integer: 
     * com.funambol.storage.ComplexSerializer.
     */
    public void serializeIntegerTest() throws Exception {
        Log.info("1.Serializing Integer");
        //Serialization of Integer with ComplexSerializer
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        DataOutputStream dout = new DataOutputStream(baos);
        ComplexSerializer.serializeObject(dout, i);
        
        //Building Expected result
        ByteArrayOutputStream expectedBaos = new ByteArrayOutputStream();
        DataOutputStream expectedDout = new DataOutputStream(expectedBaos);
        expectedDout.writeByte(INTEGER);
        expectedDout.writeInt(((Integer)i).intValue());
        
        //Assert true if baos is equal to expectedBaos
        iStream = compareStream(baos, expectedBaos);
        Log.info("1.Integer Test Succesfull");
    }

    /**
     * Test of method serializeObject for String: 
     * com.funambol.storage.ComplexSerializer.
     */
    private void serializeStringTest() throws Exception {
        //Serialization of String with ComplexSerializer
        Log.info("2.Serializing String");
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        DataOutputStream dout = new DataOutputStream(baos);
        ComplexSerializer.serializeObject(dout,str);
        
        //Building Expected result
        ByteArrayOutputStream expectedBaos = new ByteArrayOutputStream();
        DataOutputStream expectedDout = new DataOutputStream(expectedBaos);
        expectedDout.writeByte(STRING);
        expectedDout.writeUTF((String)str);
        
        //Assert true if baos is equal to expectedBaos
        strStream = compareStream(baos, expectedBaos);
        Log.info("2.String Test Succesfull");
    }

    /**
     * Test of method serializeObject for Byte[]: 
     * com.funambol.storage.ComplexSerializer.
     */
    private void serializeByteArrayTest() throws Exception{
        //Serialization of byte[] with ComplexSerializer
        Log.info("3.Serializing Byte Array");
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        DataOutputStream dout = new DataOutputStream(baos);
        ComplexSerializer.serializeObject(dout,ba);
        //Building Expected result
        ByteArrayOutputStream expectedBaos = new ByteArrayOutputStream();
        DataOutputStream expectedDout = new DataOutputStream(expectedBaos);
        expectedDout.writeByte(BYTE_ARRAY);
        expectedDout.writeInt(ba.length);
        expectedDout.write(ba, 0, ba.length);
        
        //Assert true if baos is equal to expectedBaos
        baStream = compareStream(baos, expectedBaos);
        Log.info("3.Byte Array Test Succesfull");
    }


    /**
     * Test of method serializeObject for Hashtable: 
     * com.funambol.storage.ComplexSerializer.
     */
    public void testSerializeHashTable() throws Exception {
        //Serialization of String with ComplexSerializer
        Log.info("4.Serializing HashTable");
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        DataOutputStream dout = new DataOutputStream(baos);
        ComplexSerializer.serializeHashTable(dout,ht);
        
        //Building Expected result
        ByteArrayOutputStream expectedBaos = new ByteArrayOutputStream();
        DataOutputStream expectedDout = new DataOutputStream(expectedBaos);
        
        //Building Expected result
        //expectedDout.writeByte(HASHTABLE);
        int n = ht.size();
        expectedDout.writeInt(n);
        //Writes all the elements of the Hashtable to expectedDout
        for(Enumeration e = ht.keys(); e.hasMoreElements(); ){
            String key = new String((String)e.nextElement());
            Integer val = (Integer) ht.get(key);
            //Write ht Key
            expectedDout.writeByte(STRING);
            expectedDout.writeUTF((String)key);
            //Write ht Value
            expectedDout.writeByte(INTEGER);
            expectedDout.writeInt(((Integer)val).intValue());
        }
        //Assert true if baos is equal to expectedBaos
        htStream = compareStream(baos, expectedBaos);
        Log.info("4.HashTable Test Succesfull");
    }

    /**
     * Test of method serializeObject for Vector: 
     * com.funambol.storage.ComplexSerializer.
     */
    public void testSerializeVector() throws Exception {
        Log.info("5.Serializing Vector");
        //Serialization of Integer with ComplexSerializer
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        DataOutputStream dout = new DataOutputStream(baos);
        ComplexSerializer.serializeVector(dout, v);
        
        //Building Expected result
        ByteArrayOutputStream expectedBaos = new ByteArrayOutputStream();
        DataOutputStream expectedDout = new DataOutputStream(expectedBaos);
        //expectedDout.writeByte(VECTOR);
        int n = v.size();
        expectedDout.writeInt(n);
        expectedDout.write(iStream);
        expectedDout.write(strStream);
        expectedDout.write(baStream);
        expectedDout.writeByte(HASHTABLE);
        expectedDout.write(htStream);
        
        //Assert true if baos is equal to expectedBaos
        vStream = compareStream(baos, expectedBaos);
        Log.info("5.Vector Test Succesfull");
    }

    /**
     * Test of method serializeObject for Hashtable of Vector: 
     * com.funambol.storage.ComplexSerializer.
     */
    private void testSerializeHashtableOfVector() throws Exception {
        //Serialization of String with ComplexSerializer
        Log.info("6.Serializing Hashtable with Vector and Hashtable");
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        DataOutputStream dout = new DataOutputStream(baos);
        ComplexSerializer.serializeHashTable(dout,htVectorHashtable);
        
        //Building Expected result
        ByteArrayOutputStream expectedBaos = new ByteArrayOutputStream();
        DataOutputStream expectedDout = new DataOutputStream(expectedBaos);
        
        //Building Expected result
        int n = htVectorHashtable.size();
        expectedDout.writeInt(n);
        //Writes all the elements of the Hashtable to expectedDout
        expectedDout.writeByte(VECTOR);
        expectedDout.write(vStream);
        expectedDout.writeByte(HASHTABLE);
        expectedDout.write(htStream);
        
        //Assert true if baos is equal to expectedBaos
        htvhtStream = compareStream(baos, expectedBaos);
        Log.info("6.Hashtable with Vector and Hashtable Test Successfull");
    
    }
    
    /**
     * Test of method serializeObject for Vector of Vector: 
     * com.funambol.storage.ComplexSerializer.
     */
    private void testSerializeVectorOfVector() throws Exception {
        Log.info("7.Serializing Vector with Vector and Hashtable");
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        DataOutputStream dout = new DataOutputStream(baos);
        ComplexSerializer.serializeVector(dout,vht);
        
        //Building Expected result
        ByteArrayOutputStream expectedBaos = new ByteArrayOutputStream();
        DataOutputStream expectedDout = new DataOutputStream(expectedBaos);
        
        //Building Expected result
        //expectedDout.writeByte(VECTOR);
        int n = vht.size();
        expectedDout.writeInt(n);
        
        //Writes all the elements of the Vector to expectedDout
        expectedDout.writeByte(VECTOR);
        expectedDout.write(vStream);
        expectedDout.writeByte(HASHTABLE);
        expectedDout.write(htvhtStream);
        expectedDout.writeByte(HASHTABLE);
        expectedDout.write(htStream);
        expectedDout.write(iStream);
        expectedDout.write(strStream);
        expectedDout.write(baStream);
        
        //Assert true if baos is equal to expectedBaos
        vhtStream = compareStream(baos, expectedBaos);
        Log.info("7.Vector with Vector and Hashtable Test Succesfull");
    }

    /**
     * Test of method deserializeObject for Integer: 
     * com.funambol.storage.ComplexSerializer.
     */
    private void testDeserializeInteger() throws Exception {
        Log.info("8.Deserializing Integer");
        ByteArrayInputStream bais = new ByteArrayInputStream(iStream);
        DataInputStream din = new DataInputStream(bais);
        assertEquals(ComplexSerializer.deserializeObject(din), i); 
        Log.info("8.Deserializing Integer Test Successfull");
    
    }

    /**
     * Test of method deserializeObject for String: 
     * com.funambol.storage.ComplexSerializer.
     */
    private void testDeserializeString() throws Exception {
        Log.info("9.Deserializing String");
        ByteArrayInputStream bais = new ByteArrayInputStream(strStream);
        DataInputStream din = new DataInputStream(bais);
        assertEquals(ComplexSerializer.deserializeObject(din), str); 
        Log.info("9.Deserializing String Test Successfull");
    }

    /**
     * Test of method deserializeObject for byte[]: 
     * com.funambol.storage.ComplexSerializer.
     */
    private void testDeserializeByteArray() throws Exception {
        Log.info("10.Deserializing ByteArray");
        ByteArrayInputStream bais = new ByteArrayInputStream(baStream);
        DataInputStream din = new DataInputStream(bais);
        byte[] actual = (byte[]) ComplexSerializer.deserializeObject(din);
        for (int i=0; i<actual.length; i++) {
            assertTrue(actual[i]==ba[i]); 
        }
        Log.info("10.Deserializing ByteArray Test Successfull");
    }

    /**
     * Test of method deserializeObject for HashTable: 
     * com.funambol.storage.ComplexSerializer.
     */
    private void testDeserializeHashTable() throws Exception {
        Log.info("11.Deserializing HashTable");
        ByteArrayInputStream bais = new ByteArrayInputStream(htStream);
        DataInputStream din = new DataInputStream(bais);
        assertEquals(ComplexSerializer.deserializeHashTable(din).toString(), 
                     ht.toString()); 
        Log.info("11.Deserializing HashTable Test Successfull");
    
    }

    /**
     * Test of method deserializeObject for Vector: 
     * com.funambol.storage.ComplexSerializer.
     */
    private void testDeserializeVector() throws Exception {
        Log.info("12.Deserializing Vector");
        ByteArrayInputStream bais = new ByteArrayInputStream(vStream);
        DataInputStream din = new DataInputStream(bais);
        Vector actual = new Vector();
        actual = (Vector) ComplexSerializer.deserializeVector(din);
        assertEquals(actual.elementAt(0), v.elementAt(0)); 
        assertEquals(actual.elementAt(1), v.elementAt(1)); 
        byte[] actualVectorElement = (byte[]) v.elementAt(2);
        for (int i=0; i<actualVectorElement.length; i++) {
            assertTrue(actualVectorElement[i]==ba[i]); 
        }
        assertEquals(actual.elementAt(3).toString(), v.elementAt(3).toString()); 
        Log.info("12.Deserializing Vector Test Successfull");
    }
    
    /**
     * Test of method deserializeObject for HashtableOfVector: 
     * com.funambol.storage.ComplexSerializer.
     */
    private void testDeserializeHashtableOfVector() throws Exception {
        Log.info("13.Deserializing HashtableOfVector");
        ByteArrayInputStream bais = new ByteArrayInputStream(htvhtStream);
        DataInputStream din = new DataInputStream(bais);
        //v, ht
        Hashtable actual = new Hashtable();
        actual = (Hashtable) ComplexSerializer.deserializeHashTable(din); 
        Enumeration actualEnum = actual.keys();
        Enumeration expectedEnum = htVectorHashtable.keys(); 
        while (actualEnum.hasMoreElements()) {
            assertEquals(actualEnum.nextElement().getClass().getName(), 
                         expectedEnum.nextElement().getClass().getName());
        }
        
        while (actualEnum.hasMoreElements()) {
            assertEquals(actualEnum.nextElement().toString(), 
                         expectedEnum.nextElement().toString());
        }
        
        actualEnum = actual.elements();
        expectedEnum = htVectorHashtable.elements();
        
        while (actualEnum.hasMoreElements()) {
            assertEquals(actualEnum.nextElement().getClass().getName(), 
                         expectedEnum.nextElement().getClass().getName());
        }
        
        while (actualEnum.hasMoreElements()) {
            assertEquals(actualEnum.nextElement().toString(), 
                         expectedEnum.nextElement().toString());
        }
        
        assertTrue(actual.size()==1);
        
        
        //htVectorHashtable; 
        Log.info("13.Deserializing HashtableOfVector Test Successfull");
    }

    private void testDeserializeVectorOfVector() throws Exception {
        Log.info("14.Deserializing VectorOfVector");
        ByteArrayInputStream bais = new ByteArrayInputStream(vhtStream);
        DataInputStream din = new DataInputStream(bais);
        
        Vector actual = new Vector(); 
        actual = (Vector) ComplexSerializer.deserializeVector(din);
        assertTrue(actual.size()==6);
        assertTrue(actual.elementAt(0) instanceof Vector); 
        assertTrue(actual.elementAt(1) instanceof Hashtable); 
        assertTrue(actual.elementAt(2) instanceof Hashtable); 
        assertTrue(actual.elementAt(3) instanceof Integer); 
        assertTrue(actual.elementAt(4) instanceof String); 
        assertTrue(actual.elementAt(5) instanceof byte[]); 
        
        assertEquals(actual.elementAt(2).toString(), ht.toString());
        assertEquals(actual.elementAt(3), i);
        assertEquals(actual.elementAt(4), str);
        
        byte[] actualBa = (byte[]) actual.elementAt(5);
        for (int i=0; i<actualBa.length; i++) {
            assertTrue(actualBa[i]==ba[i]);
        }
        
        Log.info("14.Deserializing VectorOfVector Test Successfull");
    }
    
    /**
     * Build various private object in order to fully accamplish TestCase 
     * com.funambol.storage.ComplexSerializer.java
     */
    private void createObjects() {
        i = new Integer(100);
        
        str = new String("String to be retrieved");
        
        ba = new byte[3];
        ba[0] = 1;
        ba[1] = 2;
        ba[2] = 3;
        
        ht = new Hashtable();
        ht.put(new String("key 0"), new Integer(0));
        ht.put(new String("key 1"), new Integer(1));
        ht.put(new String("key 2"), new Integer(2));
        ht.put(new String("key 3"), new Integer(3));
        
        v = new Vector();
        v.addElement(i);
        v.addElement(str);
        v.addElement(ba);
        v.addElement(ht);
        
        htVectorHashtable = new Hashtable();
        htVectorHashtable.put(v, ht);
        
        vht = new Vector();
        vht.addElement(v);
        vht.addElement(htVectorHashtable);
        vht.addElement(ht);
        vht.addElement(i);
        vht.addElement(str);
        vht.addElement(ba);
        
        SerializeMe sm = new SerializeMe();
        serializableHt = new Hashtable();
        serializableHt.put(sm, sm);
        
    }

    /**
     * Compare actual and expected Byte array byte by byte.
     * @param baos is the ByteArrayOutputStream obtained from serialization
     * @param expectedBaos is the expected ByteArrayOutputStream
     * @return byte[] result for next test.
     */
    private byte[] compareStream(ByteArrayOutputStream baos, 
                               ByteArrayOutputStream expectedBaos) 
                               throws AssertionFailedException {
        byte[] actual = expectedBaos.toByteArray();
        byte[] expected = baos.toByteArray();
        for (int j=0; j<actual.length; j++) {
            assertTrue(actual[j]==expected[j]);
        }
        //Return the byteArray to use for deserialization test 
        return actual;
    }
    
    /**
     * Test of method serializeObject for Vector of Vector: 
     * com.funambol.storage.ComplexSerializer.
     */
    private void testSerializeSerializableHashtable() throws Exception {
        Log.info("15.Serializing Hashtable with Serialized Elements");
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        DataOutputStream dout = new DataOutputStream(baos);
        ComplexSerializer.serializeHashTable(dout,serializableHt);
        
        //Building Expected result
        ByteArrayOutputStream expectedBaos = new ByteArrayOutputStream();
        DataOutputStream expectedDout = new DataOutputStream(expectedBaos);
        
        //Building Expected result
        //expectedDout.writeByte(VECTOR);
        expectedDout.writeInt(serializableHt.size());
        
        //Writes all the elements of the Vector to expectedDout
        expectedDout.writeByte(SERIALIZED);
        expectedDout.writeUTF((new SerializeMe()).getClass().getName());
        expectedDout.writeUTF("Serializing me");
        expectedDout.writeByte(SERIALIZED);
        expectedDout.writeUTF((new SerializeMe()).getClass().getName());
        expectedDout.writeUTF("Serializing me");
        
        //Assert true if baos is equal to expectedBaos
        serializableHtStream = compareStream(baos, expectedBaos);
        Log.info("15.Serializing Hashtable with Serialized Elements");
    }
    
    private void deserializeSerializableHashtable() throws Exception{
        Log.info("16.Deserializing Hashtable with Serialized Elements");
        ByteArrayInputStream bais = new ByteArrayInputStream(serializableHtStream);
        DataInputStream din = new DataInputStream(bais);
        
        Hashtable actual = new Hashtable();
        actual = (Hashtable) ComplexSerializer.deserializeHashTable(din);
        assertTrue(actual.size()==1);
        
        Log.info(actual.toString());
        Log.info("16.Deserializing Hashtable with Serialized Elements " +
                                                                  "succesfull");
    }

}
